#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <straight/straight.h>

#define DEBUG 1

char inputAFileName[256] = "";
char inputBFileName[256] = "";
char outputFileName[256] = "";

char outputPluginName[256] = "output_wav";

static stBool callbackFunc(Straight straight, stCallbackType callbackType,
                           void *callbackData, void *userData)
{
    int percent;
    
    if (callbackType == STRAIGHT_F0_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT F0: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_AP_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT AP: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SPECGRAM_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT spectrogram: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SYNTH_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT synthesis: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    }

    return ST_TRUE;
}

int main(int argc, char *argv[])
{
    long n, k;
    long frequencyLength;
    long nframesA, nframesB, nframes;
    StraightConfig config;
    Straight straight;
    StraightSource sourceA, sourceB, sourceMorph;
    StraightSpecgram specgramA, specgramB, specgramMorph;
    StraightSynth synth;
    double *inputAWave, *inputBWave;
    long inputAWaveLength, inputBWaveLength;
    double f0A, f0B, f0Morph;
    double *apBufferPtrA, *apBufferPtrB, *apBufferPtrMorph;
    double *specBufferPtrA, *specBufferPtrB, *specBufferPtrMorph;

    if (argc < 4) {
        fprintf(stderr, "simple <input file A> <input file B> <output file>\n");
        exit(1);
    }
    
    strcpy(inputAFileName, argv[1]);
    strcpy(inputBFileName, argv[2]);
    strcpy(outputFileName, argv[3]);

#ifdef USE_TANDEM
    straightInitConfigTandem(&config);
#else
    straightInitConfig(&config);
#endif
    config.samplingFrequency = 16000.0;

    if ((straight = straightInitialize(&config)) != NULL) {
        if (straightReadAudioFile(straight, inputAFileName, NULL) == ST_FALSE) {
            fprintf(stderr, "Error: Cannot open file: %s\n", inputAFileName);
        } else {
            inputAWave = xstraightGetCurrentWave(straight, &inputAWaveLength);

            if (straightReadAudioFile(straight, inputBFileName, NULL) == ST_FALSE) {
                fprintf(stderr, "Error: Cannot open file: %s\n", inputBFileName);
            } else {
                inputBWave = xstraightGetCurrentWave(straight, &inputBWaveLength);

                fprintf(stderr, "Input file A: %s\n", inputAFileName);
                fprintf(stderr, "Input file B: %s\n", inputBFileName);
            
                straightSetCallbackFunc(straight, STRAIGHT_PERCENTAGE_CALLBACK, callbackFunc, NULL);
                
                sourceA = straightSourceInitialize(straight, NULL);
                straightSourceCompute(straight, sourceA, inputAWave, inputAWaveLength);
                
                specgramA = straightSpecgramInitialize(straight, NULL);
                straightSpecgramCompute(straight, sourceA, specgramA, inputAWave, inputAWaveLength);

                sourceB = straightSourceInitialize(straight, NULL);
                straightSourceCompute(straight, sourceB, inputBWave, inputBWaveLength);
                
                specgramB = straightSpecgramInitialize(straight, NULL);
                straightSpecgramCompute(straight, sourceB, specgramB, inputBWave, inputBWaveLength);

                nframesA = straightSpecgramGetNumFrames(specgramA);
                nframesB = straightSpecgramGetNumFrames(specgramB);
                if (nframesA < nframesB) {
                    nframes = nframesA;
                } else {
                    nframes = nframesB;
                }
#ifdef DEBUG
                fprintf(stderr, "nframesA = %ld, nframesB = %ld, nframes = %ld\n",
                        nframesA, nframesB, nframes);
#endif
                
                sourceMorph = straightSourceInitialize(straight, NULL);
                straightSourceCreateF0(straight, sourceMorph, nframes);
                straightSourceCreateAperiodicity(straight, sourceMorph, nframes);
                
                specgramMorph = straightSpecgramInitialize(straight, NULL);
                straightSpecgramCreate(straight, specgramMorph, nframes);

                frequencyLength = straightSpecgramGetFrequencyLength(specgramMorph);
                
                for (n = 0; n < nframes; n++) {
                    straightSourceGetF0(straight, sourceA, n, &f0A);
                    straightSourceGetF0(straight, sourceB, n, &f0B);

                    /* 50% morphing of log F0 */
                    f0Morph = pow(f0A, 0.5) * pow(f0B, 0.5);
                    straightSourceSetF0(straight, sourceMorph, n, f0Morph);
                    
                    apBufferPtrA = straightSourceGetAperiodicityPointer(straight, sourceA, n);
                    apBufferPtrB = straightSourceGetAperiodicityPointer(straight, sourceB, n);
                    apBufferPtrMorph = straightSourceGetAperiodicityPointer(straight, sourceMorph, n);
                    for (k = 0; k < frequencyLength; k++) {
                        apBufferPtrMorph[k] = apBufferPtrA[k] * 0.5 + apBufferPtrB[k] * 0.5;
                    }
                    
                    specBufferPtrA = straightSpecgramGetSpectrumPointer(straight, specgramA, n);
                    specBufferPtrB = straightSpecgramGetSpectrumPointer(straight, specgramB, n);
                    specBufferPtrMorph = straightSpecgramGetSpectrumPointer(straight, specgramMorph, n);
                    for (k = 0; k < frequencyLength; k++) {
                        /* 50% morphing of log spectrum */
                        specBufferPtrMorph[k] = pow(specBufferPtrA[k], 0.5) * pow(specBufferPtrB[k], 0.5);
                    }
                }

                synth = straightSynthInitialize(straight, NULL);
                if (straightSynthCompute(straight, sourceMorph, specgramMorph, synth, 1.0, 1.0, 1.0) == ST_TRUE) {
                    straightLimitSynthWaveAmplitude(straight, synth, 0.9765625);
                    
                    if (straightWriteSynthAudioFile(straight, synth,
                                                    outputFileName, outputPluginName, 16) == ST_TRUE) {
                        fprintf(stderr, "Output file: %s\n", outputFileName);
                    } else {
                        fprintf(stderr, "Error: Cannot open file: %s\n", outputFileName);
                    }
                }
    
                straightSourceDestroy(sourceA);
                straightSourceDestroy(sourceB);
                straightSourceDestroy(sourceMorph);
                straightSpecgramDestroy(specgramA);
                straightSpecgramDestroy(specgramB);
                straightSpecgramDestroy(specgramMorph);
                straightSynthDestroy(synth);
                
                free(inputBWave);
            }
            free(inputAWave);
        }
        straightDestroy(straight);
    } else {
        fprintf(stderr, "Error: Cannot initialize STRAIGHT engine\n");
    }

    return 0;
}
