/*
 *      straight.h
 */

#ifndef __STRAIGHT_H
#define __STRAIGHT_H

#include <straight/straightDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

#if defined(MACOS)
#pragma import on
#endif

extern long straightGetEngineVersionIdFromString(char *versionString); /* 1.1.0+ */
extern char *straightGetEngineVersionStringFromId(long versionId, stBool shortFlag); /* 1.1.0+ */
    
extern stBool _straightInitConfig(StraightConfig *config, unsigned long versionId, unsigned long defaultEngineVersionId);
#define straightInitConfig(conf) _straightInitConfig(conf, STRAIGHT_VERSION_ID, STRAIGHT_ENGINE_VERSION_ID)
#define straightInitConfigV30(conf) _straightInitConfig(conf, STRAIGHT_VERSION_ID, STRAIGHT_V30_ENGINE_VERSION_ID)
#define straightInitConfigTandem(conf) _straightInitConfig(conf, STRAIGHT_VERSION_ID, STRAIGHT_TANDEM_ENGINE_VERSION_ID)
#define straightInitConfigWithEngine(conf, engineVersionId) _straightInitConfig(conf, STRAIGHT_VERSION_ID, engineVersionId) /* 1.1.0+ */

extern Straight _straightInitialize(StraightConfig *config);
#define straightInitialize(conf) _straightInitialize((StraightConfig *)conf)
extern stBool _straightReset(Straight straight, StraightConfig *config); /* 1.1.0+ */
#define straightReset(straight, conf) _straightReset(straight, (StraightConfig *)conf)
    
extern stBool straightDestroy(Straight straight);
    
extern stBool straightSetCallbackFunc(Straight straight, stCallbackType callbackMask, stCallbackFunc func, void *userData);

extern long straightSecondToPoint(Straight straight, double second);
extern long straightMilliSecondToPoint(Straight straight, double milliSecond);
extern double straightPointToSecond(Straight straight, long point);
extern double straightPointToMilliSecond(Straight straight, long point);

extern double straightGetSamplingFrequency(Straight straight);
extern long straightGetFFTLength(Straight straight);
extern long straightGetFrameLengthInPoint(Straight straight);
extern long straightGetFrameShiftInPoint(Straight straight);
extern long straightGetF0FrameShiftInPoint(Straight straight);
    
extern stBool straightIsAudioFileSupported(char *filename, char *pluginName);
extern stBool straightReadAudioFile(Straight straight, char *filename, char *pluginName);
extern stBool straightWriteSynthAudioFile(Straight straight, StraightSynth synth,
					  char *filename, char *pluginName, int bitsPerSample);
extern stBool straightLimitSynthWaveAmplitude(Straight straight, StraightSynth synth, double limitAmplitude); /* V1.1.0+ */
    
extern stBool straightClearCurrentWave(Straight straight);
extern double *straightGetCurrentWave(Straight straight, long *waveLength);
extern double *xstraightGetCurrentWave(Straight straight, long *waveLength);
    
extern StraightAsync straightAsyncInitialize(Straight straight, stCallbackType callbackMask, stAsyncCallbackFunc func, void *userData);
extern stBool straightAsyncDestroy(StraightAsync async);
extern Straight straightAsyncGetStraight(StraightAsync async);
extern stBool straightAsyncSetWaveform(StraightAsync async, double *inputWave, long inputWaveLength);
extern stBool straightAsyncSetEvent(StraightAsync async);
extern stBool straightAsyncResetEvent(StraightAsync async);
extern stBool straightAsyncWaitEvent(StraightAsync async);

extern void *straightAsyncGetF0ThreadHandle(StraightAsync async);
extern void *straightAsyncGetAperiodicityThreadHandle(StraightAsync async);
extern void *straightAsyncGetSpecgramThreadHandle(StraightAsync async);

extern char **straightGetFFTEngineList(Straight straight, int *count);
extern stBool straightSetFFTEngine(Straight straight, char *engine);

/* 1.1.2+ */
extern char **straightGetVectorEngineList(Straight straight, int *count);
extern stBool straightSetVectorEngine(Straight straight, char *engine);
extern char *straightGetVectorEngine(Straight straight);
extern stBool straightIsVectorEngineFastModeSupported(Straight straight, stBool fastFlag);
extern stBool straightSetVectorEngineFastMode(Straight straight, stBool fastFlag);
extern stBool straightGetVectorEngineFastMode(Straight straight, stBool *fastFlag);
extern char **straightGetVectorEngineUnitList(Straight straight, int *count);
extern stBool straightSelectVectorEngineUnit(Straight straight, char *unit);
    
/* 1.1.3+ */
extern stBool straightSetParallelProcessingMode(Straight straight, stBool parallelProcessingFlag);
extern stBool straightGetParallelProcessingMode(Straight straight);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#include <straight/straightSource.h>
#include <straight/straightSpecgram.h>
#include <straight/straightSynth.h>
#include <straight/straightFile.h>

#endif /* __STRAIGHT_H */
