/*
 *      straightFile.h
 */

#ifndef __STRAIGHTFILE_H
#define __STRAIGHTFILE_H

#include <straight/straightDefs.h>
#include <straight/straightFileDefs.h>

#ifdef __cplusplus
extern "C" {
#endif
    
#if defined(MACOS)
#pragma import on
#endif

extern StraightFile straightFileOpen(char *filename, char *mode);
extern stBool straightFileClose(StraightFile sf);

/* straightFileSet* must be called before straightFileWriteHeader */
extern stBool straightFileSetNextFile(StraightFile sf, char *nextFile);
extern char *straightFileGetNextFile(StraightFile sf);
extern stBool straightFileSetChunkList(StraightFile sf, char *chunkList);
extern char *straightFileGetChunkList(StraightFile sf);
extern stBool straightFileSetEndian(StraightFile sf, stBool bigEndianFlag);
extern stBool straightFileGetEndian(StraightFile sf, stBool *bigEndianFlag);
    
extern stBool straightFileLoadHeader(StraightFile sf, Straight straight);
extern stBool straightFileLoadChunkInfo(StraightFile sf, char *chunkId, long *chunkSize);
extern stBool straightFileSkipChunk(StraightFile sf, long chunkSize);

/* the following 6 loading functions must be called just after each straightFileLoadChunkInfo */    
extern stBool straightFileLoadF0(StraightFile sf, StraightSource source);
extern stBool straightFileLoadUnvoicedF0(StraightFile sf, StraightSource source);
extern stBool straightFileLoadPeriodicityLevel(StraightFile sf, StraightSource source);
extern stBool straightFileLoadF0Candidates(StraightFile sf, StraightSource source);
extern stBool straightFileLoadAperiodicity(StraightFile sf, StraightSource source);
extern stBool straightFileLoadSpecgram(StraightFile sf, StraightSpecgram specgram);

extern stBool straightFileWriteHeader(StraightFile sf, Straight straight);
extern stBool straightFileWriteF0Chunk(StraightFile sf, StraightSource source);
extern stBool straightFileWriteUnvoicedF0Chunk(StraightFile sf, StraightSource source);
extern stBool straightFileWritePeriodicityLevelChunk(StraightFile sf, StraightSource source);
extern stBool straightFileWriteF0CandidatesChunk(StraightFile sf, StraightSource source);
extern stBool straightFileWriteAperiodicityChunk(StraightFile sf, StraightSource source);
extern stBool straightFileWriteSpecgramChunk(StraightFile sf, StraightSpecgram specgram);
    
#if defined(MACOS)
#pragma import off
#endif
    
#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __STRAIGHTFILE_H */
