#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <straight/straight.h>

char inputFileName[256] = "";
char outputFileName[256] = "";

char outputPluginName[256] = "output_wav";

static stBool callbackFunc(Straight straight, stCallbackType callbackType,
                           void *callbackData, void *userData)
{
    int percent;
    
    if (callbackType == STRAIGHT_F0_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT F0: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_AP_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT AP: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SPECGRAM_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT spectrogram: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SYNTH_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT synthesis: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    }

    return ST_TRUE;
}

int main(int argc, char *argv[])
{
    StraightConfig config;
    Straight straight;
    StraightSource source;
    StraightSpecgram specgram;
    StraightSynth synth;

    if (argc < 3) {
        fprintf(stderr, "simple <input file> <output file>\n");
        exit(1);
    }
    
    strcpy(inputFileName, argv[1]);
    strcpy(outputFileName, argv[2]);

#ifdef USE_TANDEM
    straightInitConfigTandem(&config);
#else
    straightInitConfig(&config);
#endif
    config.samplingFrequency = 16000.0;

    if ((straight = straightInitialize(&config)) != NULL) {
#if defined(FFT_ENGINE)
        fprintf(stderr, "FFT engine: %s\n", FFT_ENGINE);
        if (!straightSetFFTEngine(straight, FFT_ENGINE)) {
            fprintf(stderr, "straightSetFFTEngine to %s failed\n", FFT_ENGINE);
        }
#endif
        
        if (straightReadAudioFile(straight, inputFileName, NULL) == ST_FALSE) {
            fprintf(stderr, "Error: Cannot open file: %s\n", inputFileName);
        } else {
            fprintf(stderr, "Input file: %s\n", inputFileName);
            
            straightSetCallbackFunc(straight, STRAIGHT_PERCENTAGE_CALLBACK, callbackFunc, NULL);

            source = straightSourceInitialize(straight, NULL);
            straightSourceCompute(straight, source, NULL, 0);

            specgram = straightSpecgramInitialize(straight, NULL);
            straightSpecgramCompute(straight, source, specgram, NULL, 0);

            synth = straightSynthInitialize(straight, NULL);
            if (straightSynthCompute(straight, source, specgram, synth, 1.0, 1.0, 1.0) == ST_TRUE) {
                straightLimitSynthWaveAmplitude(straight, synth, 0.9765625);
                
                if (straightWriteSynthAudioFile(straight, synth,
                                                outputFileName, outputPluginName, 16) == ST_TRUE) {
                    fprintf(stderr, "Output file: %s\n", outputFileName);
                } else {
                    fprintf(stderr, "Error: Cannot open file: %s\n", outputFileName);
                }
            }
    
            straightSourceDestroy(source);
            straightSpecgramDestroy(specgram);
            straightSynthDestroy(synth);
        }
        straightDestroy(straight);
    } else {
        fprintf(stderr, "Error: Cannot initialize STRAIGHT engine\n");
    }

    return 0;
}
